CREATE OR REPLACE FUNCTION protis_add_interval(
    p_date      TIMESTAMP,
    p_interval  INT,
    p_units     VARCHAR(10)
) RETURNS timestamp
AS $$
    DECLARE result TIMESTAMP;
BEGIN

    CASE p_units
        WHEN 'seconds' THEN result = p_date + p_interval * INTERVAL '1 second';
        WHEN 'minutes' THEN result = p_date + p_interval * INTERVAL '1 minute';
        WHEN 'hours'   THEN result = p_date + p_interval * INTERVAL '1 hour';
        WHEN 'days'    THEN result = p_date + p_interval * INTERVAL '1 day';
        WHEN 'months'  THEN result = p_date + p_interval * INTERVAL '1 month';
        WHEN 'years'   THEN result = p_date + p_interval * INTERVAL '1 year';
    END CASE;

    RETURN result;
END;
$$ LANGUAGE plpgsql;


CREATE OR REPLACE FUNCTION protis_timestamp_diff_sec(
    p_date1     TIMESTAMP WITH TIME ZONE,
    p_date2     TIMESTAMP WITH TIME ZONE
) RETURNS INT
AS $$
    DECLARE t INTERVAL;
    DECLARE d INT;
    DECLARE h INT;
    DECLARE m INT;
    DECLARE s INT;
BEGIN
    t := p_date2 - p_date1;

    h = EXTRACT(hour FROM t);
    m = EXTRACT(minute FROM t);
    s = ROUND(EXTRACT(second FROM t));
    d = EXTRACT(day FROM t);

    RETURN s + m * 60 + h * 3600 + d * 86400;
END;
$$ LANGUAGE plpgsql;


CREATE OR REPLACE FUNCTION protis_timestamp_diff_str(
    p_date1     TIMESTAMP WITH TIME ZONE,
    p_date2     TIMESTAMP WITH TIME ZONE
) RETURNS VARCHAR
AS $$
    DECLARE t INTERVAL;
    DECLARE d INT;
    DECLARE h VARCHAR;
    DECLARE m VARCHAR;
    DECLARE s VARCHAR;
    DECLARE st VARCHAR;
    DECLARE sign VARCHAR;
BEGIN
    IF p_date2 > p_date1 THEN
        t := p_date2 - p_date1;
        sign = '';
    ELSE
        t := p_date1 - p_date2;
        sign = '-';
    END IF;

    h = EXTRACT(hour FROM t);
    m = EXTRACT(minute FROM t);
    s = ROUND(EXTRACT(second FROM t));
    d = EXTRACT(day FROM t);

    st = LPAD(h, 2, '0') || ':' || LPAD(m, 2, '0') || ':' || LPAD(s, 2, '0');
    IF d = 0 THEN
        RETURN sign || st;
    ELSEIF d = 1 THEN
        RETURN sign || '1 day ' || st;
    END IF;

    RETURN sign || d || ' days ' || st;
END;
$$ LANGUAGE plpgsql;
